-- Legal_Objects_Script
use	Legal
;
go

exec	sp_configure
		'clr enabled'
		,1
;
reconfigure
;
go
	
if	(schema_id('Utility') is null)
	begin
		exec	(N'
			create schema	Utility
				authorization	dbo
			;
			')
	end
go

if	(schema_id('Taxonomy') is null)
	begin
		exec	(N'
			create schema	Taxonomy
				authorization	dbo
			;
			')
	end
go

if	(schema_id('Document') is null)
	begin
		exec	(N'
			create schema	Document
				authorization	dbo
			;
			')
	end
go

if	(object_id('Document.Fact') is not null)
	begin
		drop table	Document.Fact
		;
	end
go

if	(object_id('Document.Law') is not null)
	begin
		drop table	Document.Law
		;
	end
go

if	(object_id('Taxonomy.LawType') is not null)
	begin
		drop table	Taxonomy.LawType
		;
	end
go

if	(object_id('Utility.SystemValidUntil') is not null)
	begin
		drop function	Utility.SystemValidUntil
	end
go

if	(object_id('Utility.SystemValidSince') is not null)
	begin
		drop function	Utility.SystemValidSince
	end
go

if	(exists (
		select	*
			from	sys.assemblies
			where	(assemblies.name = 'Utility')
		))
	begin
		drop assembly	Utility
	end
go

create assembly	Utility
from	N'C:\SQLCLR\Legal.Utility.SqlClr.dll'
	with	permission_set = safe
;
go

create function	Utility.SystemValidSince
	(
	@ValidSince				date							= null
	)
returns	datetime2(7)
as external name	Utility.[Legal.Utility.System].ValidSince
;
go

create function	Utility.SystemValidUntil
	(
	@ValidUntil				date							= null
	)
returns	datetime2(7)
as external name	Utility.[Legal.Utility.System].ValidUntil
;
go

create table	Taxonomy.LawType
	(
	LawTypeId				int					identity(1, 1)	not null
	,Name					sysname							not null
	,constraint	pk_Taxonomy_LawType
			primary key clustered
			(
			LawTypeId
			)
	,constraint	uq_Taxonomy_LawType_Name
			unique nonclustered
			(
			Name
			)
	)

create table	Document.Law
	(
	LawId					int					identity(1, 1)	not null
	,Name					sysname							not null
	,Title					nvarchar(max)						not null
	,VersionNumber				smallint						not null
	,PublishedOn				date							not null
	,ValidSince				date							null
	,ValidUntil				date							null
	,RuleText				nvarchar(max)						not null
	,LawTypeId				int							not null
	,SystemValidSince
		as
		(
		Utility.SystemValidSince
			(
			ValidSince
			)
		)
		persisted
	,SystemValidUntil
		as
		(
		Utility.SystemValidUntil
			(
			ValidUntil
			)
		)
		persisted
	,constraint	pk_Document_Law
			primary key clustered
			(
			LawId
			)
	,constraint	uq_Document_Law_Name_VersionNumber
			unique nonclustered
			(
			Name
			,VersionNumber
			)
	,constraint	fk_Document_Law_LawTypeId_Taxonomy_LawType
			foreign key
			(
			LawTypeId
			)
			references	Taxonomy.LawType
			(
			LawTypeId
			)
	,constraint	chk_Document_Law_SystemValidDate
			check
			(
			SystemValidUntil > SystemValidSince
			)
	)
;
go

create unique nonclustered index	ux_Document_Law_SystemValidPeriod
	on	Document.Law
		(
		Name
		,VersionNumber
		,SystemValidSince
		,SystemValidUntil
		)
;
go

create table	Document.Fact
	(
	FactId					int							identity(1, 1)	not null
	,CaseNumber				int									not null
	,CaseYear				smallint								not null
	,Name					sysname									not null
	,Abstract				nvarchar(max)								not null
	,RelevantSince				date									not null
	,RelevantUntil				date									null
	,RelevantLawTypeId			int									not null
	,SystemRelevantSince
		as
		(
		Utility.SystemValidSince
			(
			RelevantSince
			)
		)
		persisted
	,SystemRelevantUntil
		as
		(
		Utility.SystemValidUntil
			(
			RelevantUntil
			)
		)
		persisted
	,constraint	pk_Document_Fact
			primary key clustered
			(
			FactId
			)
	,constraint	uq_Document_Fact_CaseInformation
			unique nonclustered
			(
			CaseNumber
			,CaseYear
			,Name
			,RelevantSince
			,RelevantUntil
			)
	,constraint	chk_Document_Fact_SystemRelevantDate
			check
			(
			SystemRelevantUntil > SystemRelevantSince
			)
	)
go

create unique nonclustered index	ux_Document_Fact_SystemRelevantPeriod
	on	Document.Fact
		(
		CaseNumber
		,CaseYear
		,Name
		,RelevantLawTypeId
		,SystemRelevantSince
		,SystemRelevantUntil
		)
;
go
