-- Legal_Document_RelevantLaw_Get_byCase_Proc
if	(object_id('Document.RelevantLaw_Get_byCase') is not null)
	begin
		drop proc	Document.RelevantLaw_Get_byCase
	end
go

create proc	Document.RelevantLaw_Get_byCase
	(
	@CaseNumber				int
	,@CaseYear				smallint
	)
as
begin
	set	nocount	on

	begin
		with	PrincipalMaterialLaw
		as
		(
		select	RelevantLaw.*
			,Fact.Name as FactName
			,Fact.RelevantSince as FactRelevantSince
			,Fact.RelevantUntil as FactRelevantUntil
			,FactDuration
				= datediff
					(
					day
					,Fact.SystemRelevantSince
					,case
						when	(Fact.RelevantUntil is null)
							then	getdate()
						else	Fact.SystemRelevantUntil
						end
					)
				+ 1
			,Reason
				= N'principal '
				+ LawType.Name
				+ N' law '
				+ Fact.Relevance
			from	Document.Law RelevantLaw
				inner join	Taxonomy.LawType
						on	LawType.LawTypeId = RelevantLaw.LawTypeId
				cross apply	(
						-- Material Law valid at Material Fact beginning.
						select	*
							,N'valid' as Relevance
							from	Document.Fact
							where	(Fact.RelevantLawTypeId = RelevantLaw.LawTypeId)
								and (Fact.SystemRelevantSince between RelevantLaw.SystemValidSince and RelevantLaw.SystemValidUntil)
						union
						-- Material Law valid at Material Fact end.
						select	*
							,N'valid' as Relevance
							from	Document.Fact
							where	(Fact.RelevantLawTypeId = RelevantLaw.LawTypeId)
								and (Fact.SystemRelevantUntil between RelevantLaw.SystemValidSince and RelevantLaw.SystemValidUntil)
						union
						-- Material Law valid during Material Fact.
						select	*
							,N'valid' as Relevance
							from	Document.Fact
							where	(Fact.RelevantLawTypeId = RelevantLaw.LawTypeId)
								and (RelevantLaw.SystemValidSince between Fact.SystemRelevantSince and Fact.SystemRelevantUntil)
						) Fact
			where	(LawType.Name = N'material')
				and (Fact.CaseNumber = @CaseNumber)
				and (Fact.CaseYear = @CaseYear)
		)
		,SubsidiaryMaterialLaw
		as
		(
		select	RelevantLaw.*
			,Fact.Name as FactName
			,Fact.RelevantSince as FactRelevantSince
			,Fact.RelevantUntil as FactRelevantUntil
			,FactDuration
				= datediff
					(
					day
					,Fact.SystemRelevantSince
					,case
						when	(Fact.RelevantUntil is null)
							then	getdate()
						else	Fact.SystemRelevantUntil
						end
					)
				+ 1
			,Reason
				= N'subsidiary '
				+ LawType.Name
				+ N' law '
				+ Fact.Relevance
			from	Document.Law RelevantLaw
				inner join	Taxonomy.LawType
						on	LawType.LawTypeId = RelevantLaw.LawTypeId
				inner join	PrincipalMaterialLaw
						on	(PrincipalMaterialLaw.Name = RelevantLaw.Name)
							and (PrincipalMaterialLaw.SystemValidSince < RelevantLaw.SystemValidSince)
				cross apply	(
						-- Material Law published at Material Fact beginning.
						select	*
							,N'published' as Relevance
							from	Document.Fact
							where	(Fact.RelevantLawTypeId = RelevantLaw.LawTypeId)
								and (Fact.SystemRelevantSince not between RelevantLaw.SystemValidSince and RelevantLaw.SystemValidUntil)
								and (Fact.SystemRelevantSince >= RelevantLaw.PublishedOn)
						union
						-- Material Law published at Procedural Fact beginning.
						select	*
							,N'published' as Relevance
							from	Document.Fact
							where	(Fact.RelevantLawTypeId = (
									select	SubsidiaryLawType.LawTypeId
										from	Taxonomy.LawType SubsidiaryLawType
										where	(SubsidiaryLawType.Name = N'procedural')
									))
								and (Fact.SystemRelevantSince not between RelevantLaw.SystemValidSince and RelevantLaw.SystemValidUntil)
								and (Fact.SystemRelevantSince >= RelevantLaw.PublishedOn)
						) Fact
			where	(LawType.Name = N'material')
				and (Fact.CaseNumber = @CaseNumber)
				and (Fact.CaseYear = @CaseYear)
		)
		,PrincipalProceduralLaw
		as
		(
		select	RelevantLaw.*
			,Fact.Name as FactName
			,Fact.RelevantSince as FactRelevantSince
			,Fact.RelevantUntil as FactRelevantUntil
			,FactDuration
				= datediff
					(
					day
					,Fact.SystemRelevantSince
					,case
						when	(Fact.RelevantUntil is null)
							then	getdate()
						else	Fact.SystemRelevantUntil
						end
					)
				+ 1
			,Reason
				= N'principal '
				+ LawType.Name
				+ N' law '
				+ Fact.Relevance
			from	Document.Law RelevantLaw
				inner join	Taxonomy.LawType
						on	LawType.LawTypeId = RelevantLaw.LawTypeId
				cross apply	(
						-- Procedural Law valid at Procedural Fact beginning.
						select	*
							,N'valid' as Relevance
							from	Document.Fact
							where	(Fact.RelevantLawTypeId = RelevantLaw.LawTypeId)
								and (Fact.SystemRelevantSince between RelevantLaw.SystemValidSince and RelevantLaw.SystemValidUntil)
						union
						-- Procedural Law valid at Procedural Fact end.
						select	*
							,N'valid' as Relevance
							from	Document.Fact
							where	(Fact.RelevantLawTypeId = RelevantLaw.LawTypeId)
								and (Fact.SystemRelevantUntil between RelevantLaw.SystemValidSince and RelevantLaw.SystemValidUntil)
						union
						-- Procedural Law valid during Procedural Fact.
						select	*
							,N'valid' as Relevance
							from	Document.Fact
							where	(Fact.RelevantLawTypeId = RelevantLaw.LawTypeId)
								and (RelevantLaw.SystemValidSince between Fact.SystemRelevantSince and Fact.SystemRelevantUntil)
						) Fact
			where	(LawType.Name = N'procedural')
				and (Fact.CaseNumber = @CaseNumber)
				and (Fact.CaseYear = @CaseYear)
		)
		select	PrincipalMaterialLaw.FactName as FactName
			,PrincipalMaterialLaw.FactRelevantSince as FactRelevantSince
			,PrincipalMaterialLaw.FactRelevantUntil as FactRelevantUntil
			,PrincipalMaterialLaw.FactDuration as FactDuration
			,PrincipalMaterialLaw.Name as LawName
			,PrincipalMaterialLaw.VersionNumber as LawVersionNumber
			,PrincipalMaterialLaw.PublishedOn as LawPublishedOn
			,PrincipalMaterialLaw.ValidSince as LawValidSince
			,PrincipalMaterialLaw.ValidUntil as LawValidUntil
			,PrincipalMaterialLaw.Reason as Reason
			,PrincipalMaterialLaw.Title as LawTitle
			,PrincipalMaterialLaw.RuleText as LawRuleText
			from	PrincipalMaterialLaw
		union
		select	SubsidiaryMaterialLaw.FactName
			,SubsidiaryMaterialLaw.FactRelevantSince
			,SubsidiaryMaterialLaw.FactRelevantUntil
			,SubsidiaryMaterialLaw.FactDuration
			,SubsidiaryMaterialLaw.Name
			,SubsidiaryMaterialLaw.VersionNumber
			,SubsidiaryMaterialLaw.PublishedOn
			,SubsidiaryMaterialLaw.ValidSince
			,SubsidiaryMaterialLaw.ValidUntil
			,SubsidiaryMaterialLaw.Reason
			,SubsidiaryMaterialLaw.Title
			,SubsidiaryMaterialLaw.RuleText
			from	SubsidiaryMaterialLaw
		union
		select	PrincipalProceduralLaw.FactName
			,PrincipalProceduralLaw.FactRelevantSince
			,PrincipalProceduralLaw.FactRelevantUntil
			,PrincipalProceduralLaw.FactDuration
			,PrincipalProceduralLaw.Name
			,PrincipalProceduralLaw.VersionNumber
			,PrincipalProceduralLaw.PublishedOn
			,PrincipalProceduralLaw.ValidSince
			,PrincipalProceduralLaw.ValidUntil
			,PrincipalProceduralLaw.Reason
			,PrincipalProceduralLaw.Title
			,PrincipalProceduralLaw.RuleText
			from	PrincipalProceduralLaw
			order by	FactRelevantSince
					,FactRelevantUntil
	end

	return	0
end
go
