-- Legal_Demo01_Complete_Script

-- The Law
select	*
	from	Document.Law

select	ResultsToText
		= Law.Title
		+ N'
Version '
		+ cast(Law.VersionNumber as nvarchar(5))
		+ N'

'
		+ Law.RuleText
		+ N'


'
	from	Document.Law
	order by	Law.Name
			,Law.VersionNumber
			,Law.SystemValidSince

-- The Facts
select	*
	from	Document.Fact


-- Determining the Law to be used (Ignorantia iuris nocet!):

-- I.
-- Material Law
-- Principal material Law (Nullum crimen...)
select	Fact.*
	,RelevantLaw.*
	from	Document.Law RelevantLaw
		inner join	Taxonomy.LawType
				on	LawType.LawTypeId = RelevantLaw.LawTypeId
		cross apply	(
				-- Material Law valid at Material Fact beginning.
				select	*
					from	Document.Fact
					where	(Fact.RelevantLawTypeId = RelevantLaw.LawTypeId)
						and (Fact.SystemRelevantSince between RelevantLaw.SystemValidSince and RelevantLaw.SystemValidUntil)
				union
				-- Material Law valid at Material Fact end.
				select	*
					from	Document.Fact
					where	(Fact.RelevantLawTypeId = RelevantLaw.LawTypeId)
						and (Fact.SystemRelevantUntil between RelevantLaw.SystemValidSince and RelevantLaw.SystemValidUntil)
				union
				-- Material Law valid during Material Fact interval.
				select	*
					from	Document.Fact
					where	(Fact.RelevantLawTypeId = RelevantLaw.LawTypeId)
						and (RelevantLaw.SystemValidSince between Fact.SystemRelevantSince and Fact.SystemRelevantUntil)
				) Fact
	where	(LawType.Name = N'material')
	order by	Fact.SystemRelevantSince
go

-- Subsidiary material law (In dubio...)
with	PrincipalMaterialLaw
as
(
select	RelevantLaw.*
	from	Document.Law RelevantLaw
		inner join	Taxonomy.LawType
				on	LawType.LawTypeId = RelevantLaw.LawTypeId
		cross apply	(
				-- Material Law valid at Material Fact beginning.
				select	*
					from	Document.Fact
					where	(Fact.RelevantLawTypeId = RelevantLaw.LawTypeId)
						and (Fact.SystemRelevantSince between RelevantLaw.SystemValidSince and RelevantLaw.SystemValidUntil)
				union
				-- Material Law valid at Material Fact end.
				select	*
					from	Document.Fact
					where	(Fact.RelevantLawTypeId = RelevantLaw.LawTypeId)
						and (Fact.SystemRelevantUntil between RelevantLaw.SystemValidSince and RelevantLaw.SystemValidUntil)
				union
				-- Material Law valid during Material Fact interval.
				select	*
					from	Document.Fact
					where	(Fact.RelevantLawTypeId = RelevantLaw.LawTypeId)
						and (RelevantLaw.SystemValidSince between Fact.SystemRelevantSince and Fact.SystemRelevantUntil)
				) Fact
	where	(LawType.Name = N'material')
)
select	Fact.*
	,RelevantLaw.*
	from	Document.Law RelevantLaw
		inner join	Taxonomy.LawType
				on	LawType.LawTypeId = RelevantLaw.LawTypeId
		inner join	PrincipalMaterialLaw
				on	(PrincipalMaterialLaw.Name = RelevantLaw.Name)
					and (PrincipalMaterialLaw.SystemValidSince < RelevantLaw.SystemValidSince)
		cross apply	(
				-- Material Law published at Material Fact beginning.
				select	*
					from	Document.Fact
					where	(Fact.RelevantLawTypeId = RelevantLaw.LawTypeId)
						and (Fact.SystemRelevantSince not between RelevantLaw.SystemValidSince and RelevantLaw.SystemValidUntil)
						and (Fact.SystemRelevantSince >= RelevantLaw.PublishedOn)
				union
				-- Material Law published at Procedural Fact beginning.
				select	*
					from	Document.Fact
					where	(Fact.RelevantLawTypeId = (
							select	SubsidiaryLawType.LawTypeId
								from	Taxonomy.LawType SubsidiaryLawType
								where	(SubsidiaryLawType.Name = N'procedural')
							))
						and (Fact.SystemRelevantSince not between RelevantLaw.SystemValidSince and RelevantLaw.SystemValidUntil)
						and (Fact.SystemRelevantSince >= RelevantLaw.PublishedOn)
				) Fact
	where	(LawType.Name = N'material')
	order by	Fact.SystemRelevantSince
go

-- II.
-- Procedural law

-- Principal procedural law
select	Fact.*
	,RelevantLaw.*
	from	Document.Law RelevantLaw
		inner join	Taxonomy.LawType
				on	LawType.LawTypeId = RelevantLaw.LawTypeId
		cross apply	(
				-- Procedural Law valid at Procedural Fact beginning.
				select	*
					from	Document.Fact
					where	(Fact.RelevantLawTypeId = RelevantLaw.LawTypeId)
						and (Fact.SystemRelevantSince between RelevantLaw.SystemValidSince and RelevantLaw.SystemValidUntil)
				union
				-- Procedural Law valid at Procedural Fact end.
				select	*
					from	Document.Fact
					where	(Fact.RelevantLawTypeId = RelevantLaw.LawTypeId)
						and (Fact.SystemRelevantUntil between RelevantLaw.SystemValidSince and RelevantLaw.SystemValidUntil)
				union
				-- Procedural Law valid during Procedural Fact interval.
				select	*
					from	Document.Fact
					where	(Fact.RelevantLawTypeId = RelevantLaw.LawTypeId)
						and (RelevantLaw.SystemValidSince between Fact.SystemRelevantSince and Fact.SystemRelevantUntil)
				) Fact
	where	(LawType.Name = N'procedural')
go


-- Scenario 1
declare	@CaseNumber				int
declare	@CaseYear				smallint

set	@CaseNumber = 278
set	@CaseYear = 1999

exec	Document.RelevantLaw_Get_byCase
		@CaseNumber = @CaseNumber
		,@CaseYear = @CaseYear
go


-- Scenario 2
declare	@CaseNumber				int
declare	@CaseYear				smallint

set	@CaseNumber = 23
set	@CaseYear = 2003

exec	Document.RelevantLaw_Get_byCase
		@CaseNumber = @CaseNumber
		,@CaseYear = @CaseYear
go


-- Scenario 3
declare	@CaseNumber				int
declare	@CaseYear				smallint

set	@CaseNumber = 551
set	@CaseYear = 2005

exec	Document.RelevantLaw_Get_byCase
		@CaseNumber = @CaseNumber
		,@CaseYear = @CaseYear
go
